var AppModel = Backbone.Model.extend({
	URL_AFTER_INSTALL: 'https://www.couponcabin.com/?' + UTM_TAIL,
	links: new Links(),
	merchants: new MerchantCollection(),
	user: new User(),
	blockedSliders: [],
	initialize: function() {
		var this_ = this;
		this.listenTo(this, 'dont-show-slider', this.dontShowSlider);
		//this.listenTo(this, 'get-user-data', this.getUserData);
		this_.on({
			'after-install': function() {
				framework.extension.fireEvent('google-analytics-event', {
					data: {
						uid: this_.get('uid'),
						eventCategory: 'extension_activity',
						eventAction: 'install'
					}
				});
				
				this_.redirect(this.URL_AFTER_INSTALL, framework.browser.NEWTAB);
			},
			'before-navigate': function(event) {
				this_.links.match(event.url, event.tabId);
			},
			'check-url': function(event) {
				var state = this_.links.state(event.tabId);
				if (state.activation || state.suppress) {
					var merchant = this_.merchants.match(getDomain(event.url));
					if (merchant.get('has_cash_back')) {
						if (state.activation) {
							merchant.set('state', 'activated');
						} else if (state.suppress) {
							merchant.set('state', 'suppressed');
						}
						merchant.visit();
						this_.links.reset(event.tabId);
					}
				}
			},
			'change:usertoken': function() {
				this_.save();
				
				this.merchants.each(function(merchant) {
					merchant.reset();
				});
				
				framework.extension.fireEvent('reload', {
					tabId: framework.browser.ALLTABS,
					data: {}
				});
			}
		});
		
		this_.popup();
		
		this.merchants.fetch();
		// this.user.fetch();

		framework.extension.getItem('blockedSliders', function(blockedSliders) {
			if (blockedSliders) {
				this_.blockedSliders = blockedSliders.split(' ');
			}
		});
	},
	
	getCurrentMerchant: function() {
		return this.merchants.match(getDomain(this.get('url')));
	},
	
	redirect: function(url, tabId) {
		framework.browser.navigate({
			url: url,
			tabId: tabId
		});
	},
	
	sync: function(method, model) {
		var this_ = this;
		switch(method) {
			case 'read':
				framework.extension.getItem('App', function(model) {
					if (model) {
						this_.set(model);
					} else {
						this_.save({
							uid: Math.floor(Math.random(1)*1e9),
							usertoken: false
						});
						this_.trigger('after-install');
					}
				});
				break;
			case 'create': 
				framework.extension.setItem('App', this_.toJSON());
				break;
		}
	},
	
	log: function() {},
	
	singInPopup: function(url, width, height, tabId) {
		var left = Math.floor((window.screen.availWidth - width) / 2);
		var top = Math.floor((window.screen.availHeight - height) / 2);
		if ((framework.browser.name === 'Safari' && framework.browser.version == 5) 
			|| (framework.browser.name === 'Internet Explorer')) {
			framework.extension.fireEvent('openPopup', {
				tabId: tabId,
				data: {
					url: url,
					name: 'ccWindow',
					options: 'width='+width+',height='+height+',left='+left+',top='+top
				}
			});
		} else if (framework.browser.name === 'Safari' || framework.browser.name === 'Edge') {
			framework.browser.navigate({
				url: url,
				tabId: framework.browser.NEWTAB
			});
		} else {
			var wnd = window.open(url, 'ccWindow', 'width='+width+',height='+height+',left='+left+',top='+top+',dependent');
			wnd.focus();
		}
	},

	popup: function() {
		framework.ui.button.setPopup({
			'url': 'popup/popup.html',
			'width': POPUP_WIDTH,
			'height': POPUP_HEIGHT
		});
	},

	dontShowSlider: function(event) {
		if (DEBUG) {
			console.log('dont-show-slider', event);
		}

		var this_ = this;
		var add = true;

		$(this.blockedSliders).each(function(i) {
			if (this_.blockedSliders[i] == event.data.domain_name) {
				add = false;
				return;
			}
		});

		if (add) {
			this.blockedSliders.push(event.data.domain_name);
			framework.extension.setItem('blockedSliders', this.blockedSliders.join(' '));
		}	
	},

	getUserData: function(id, callback) {
		if (DEBUG) {
			console.log('get-user-data', id);
		}

		callback(id);
	},

	'reloadAllTabs': function() {
		if (DEBUG) {
			console.log('reloadAllTabs');
		}

		framework.extension.fireEvent('reload', {
			tabId: framework.browser.ALLTABS,
			data: {}
		});
	}
});