function wc2re(wc) {
	return new RegExp(wc.replace(/\.([^\*])/g, '\\\.$1'), 'i');
}


function getDomain(url){
	if(url){
		var match = /(?:https?:\/\/)?(?:\w+:\/)?[^:?#\/\s]*?([^.\s]+\.(?:[a-z]{2,}|(?:co|org|ac|com)\.\w{2,3}))(?:[:?#\/]|$)/gi.exec(url);
		return match ? match[1].toLowerCase() : null;
	} else {
		return null;
	}
}

var AffiliateLink = Backbone.Model.extend({
	TIMEOUT: 60000, // 60 seconds
	constructor: function(url) {
		if (typeof url === 'string') {
			url = wc2re(url);
		}
		return Backbone.Model.call(this, {'url': url});
	},
	activate: function(tabId) {
		var this_ = this;
		this_.set({'tabId': tabId});
		window.setTimeout(function() {
			this_.unset('tabId');
		}, this_.TIMEOUT);
	}
});

var LinksCollection = Backbone.Collection.extend({
	model: AffiliateLink,
	initialize: function(links) {
		this.set(links);
	},
	match: function(url, tabId) {
		var matchedAffiliate = this.find(function(link) {
			return link.get('url').test(url);
		});
		if (matchedAffiliate) {
			matchedAffiliate.activate(tabId);
		}
		return matchedAffiliate;
	},
	reset: function(tabId) {
		var link = this.findWhere({
			'tabId': tabId
		});
		if (link) {
			link.unset('tabId');
		}
	},
	state: function(tabId) {
		return !!this.where({
			'tabId': tabId
		}).length;
	}
});

var Links = Backbone.Model.extend({
	suppresses: new LinksCollection(['.*commission-junction.com.*','.*qksrv.net.*','.*jdoqocy.com.*','.*tkqlhce.com.*','.*kqzyfj.com.*','.*dpbolvw.net.*','.*anrdoezrs.net.*',
		'.*google.com\/aclk.*','.*dpbolvw.net.*','.*linksynergy.com.*','.*afsrc=1.*','.*affiliate.buy.com.*','.*affiliate.rakuten.com.*','goto.target.com/c/']),
	activations: new LinksCollection([/\.couponcabin\.com\/(?:s|o)\/\d+\//i]),
	match: function(url, opt_tab) {
		var tabId = opt_tab || 0;
		this.activations.match(url, tabId);
		if (!this.activations.state(tabId)) {
			this.suppresses.match(url, tabId);
		}
	},
	state: function(opt_tab) {
		var tabId = opt_tab || 0;
		return {
			tabId: tabId,
			activation: this.activations.state(tabId),
			suppress: this.suppresses.state(tabId)
		};
	},
	reset: function(opt_tab) {
		var tabId = opt_tab || 0;
		this.suppresses.reset(tabId);
		this.activations.reset(tabId);
	}
});
