var MerchantPrototype = Backbone.Model.extend({

});

var Merchant = MerchantPrototype.extend({
	timeout: null,
	interval: 12 * 60000, // 4 hours
	initialize: function() {
		this.matchPattern = this.get('url_pattern') ? wc2re(this.get('url_pattern')) : null;
		this.on({
			'change:state': function(e, state) {
				if (state === 'activated') {
					this.set({
						'activated': true, 
						'suppressed': false, 
						'sidetab' : false
					});

					var this_ = this;

					setTimeout(function() {
						if (DEBUG) {
							console.log('activated false');
						}
						
						this_.set('state', '');

						App.reloadAllTabs();
					}, 20 * 60 * 1000); // 20 min
				} else if (state === 'suppressed') {
					this.set({
						'activated': false, 
						'suppressed': true, 
						'sidetab' : false
					});
				} else if (!state) {
					this.set({
						'activated': false, 
						'suppressed': false, 
						'sidetab' : false
					});
					
					this.unset('deal');
				}
			}
		});
	},
	reset : function() {
		this.unset('sidetab');
	},
	match: function(url) {
		var domain = url;
		return (this.get('domain_name').toLowerCase() === domain) 
			&& this.matchPattern 
			&& this.matchPattern.test(domain);
	},
	visit: function() {
		var that = this;
		if (this.timeout) {
			window.clearInterval(this.timeout);
		}
		window.setTimeout(function() {
			that.unset('state');
		}, this.interval);
	}
});

var MerchantCollection = Backbone.Collection.extend({
	UPDATE_INTERVAL: 10800000, // 3h
	ERROR_UPDATE_INTERVAL: 1000 * 60 * 10, // 10m
	url: 'https://www.couponcabin.com/api/t/',
	defaultMerchant: new MerchantPrototype(),
	timeout: null,
	model: Merchant,
	initialize: function(){
		this.on('sync', function(collection) {
			window.clearTimeout(collection.timeout);
			collection.timeout = window.setTimeout(function() {
				collection.fetch();
			}, this.UPDATE_INTERVAL);
		});
		
		this.on('error', function(collection) {
			window.clearTimeout(collection.timeout);
			collection.timeout = window.setTimeout(function() {
				collection.fetch();
			}, this.ERROR_UPDATE_INTERVAL);
		});
	},
	match: function(url) {
		if (url) {
			return this.find(function(model) {
				return model.match(url);
			}) || this.defaultMerchant;
		} else {
			return this.defaultMerchant;
		}
	},
	// fetch: function(opt_options) {
		// var this_ = this;
		// var options = opt_options || {};
		// _.extend(options, {
			// 'headers': {
				// 'X-Access-Token': 'B67D0AD2-F1EF-47A6-A888-6FF1D57E0352'
			// }
		// });
		// if(this.timeout) {
			// window.clearInterval(this.timeout);
		// }
		// this.timeout = window.setTimeout(function() {
			// this_.fetch(options);
		// }, this.UPDATE_INTERVAL);
		// return Backbone.Collection.prototype.fetch.call(this, options);
	// }, 
	sync: function(method, model, options) {
		_.extend(options, {
			'headers': {
				'X-Access-Token': 'B67D0AD2-F1EF-47A6-A888-6FF1D57E0352'
			}
		});
		return Backbone.Collection.prototype.sync.apply(this, arguments);
	}
});
