
var Tab = Backbone.Model.extend({
	
	defaults : {
		id: '',
		domain: '', 
		url: ''
	},
	
	initialize : function() {
		
		this.immediateReset = function() {
			this.unset("deeplink");
		};
		
		this.reset = _.debounce( _.bind(this.immediateReset, this), 20 * 1000 );
	}
});

var Tabs = Backbone.Collection.extend({
	
	model: Tab,
	
	initialize: function() {
		var self = this;
		
		function tabClosed(event) {
			var tab = self.findWhere({
				id: event.tabId
			});
			
			if (tab) {
				self.remove(tab);
			}
		}
		
		function tabChanged(event) {
			var id = event.tabId,
					url = event.url,
					domain = Utils.getDomain(url);
			
			var tab = self.findWhere({
				id: id
			});
			
			if (!tab) {
				//-- create new tab object
				tab = new Tab();
				
				//-- expand our collection
				self.add(tab);
				
				//-- set attributes
				tab.set({
					id: id, 
					url: url, 
					domain: domain
				});
				
				//-- if only one tab is present
				if (1 === self.length) {
					self.currentTabId = id;
					
					self.trigger(
						TAB_CHANGED,
						tab
					);
				}
			} else {
				tab.set({
					url: url, 
					domain: domain
				});
			}
		}
		
		var timeout2beforeNavigate = {};
		
		_.each(
			{
				'browser' : _.object(
					[
						framework.browser.TABCLOSED,
						framework.browser.BEFORENAVIGATE, 
						framework.browser.DOCUMENTCOMPLETE, 
						framework.browser.TABCHANGED
					],
					[
						tabClosed,
						function(event) {
							(function(tabId, url, guid) {
								timeout2beforeNavigate[guid] = window.setTimeout(
									function() {
										tabChanged({
											tabId : tabId
											, url : url
										});
									},
									BROWSER_NAME_FF === framework.browser.name ? 50 : 1
								);
								
								if (BROWSER_NAME_FF === framework.browser.name) {
									framework.extension.fireEvent(
										FIREFOX_CHECK_SCRIPTS,
										{
											tabId : tabId,
											data : url
										}, function() {
											if (timeout2beforeNavigate[guid]) {
												window.clearTimeout(timeout2beforeNavigate[guid]);
												delete timeout2beforeNavigate[guid];
											}
										}
									);
								}
							})( event.tabId, event.url, Utils.guid() );
						}, tabChanged,
						function(event) {
							tabChanged(event);
							
							self.currentTabId = event.tabId;
							
							self.trigger(
								TAB_CHANGED,
								self.selectById()
							);
						}
					]
				), 'extension' : _.object(
					[ TAB_CHANGED ],
					[ tabChanged ]
				)
			}, function(event2callback, context) {
				_.each(event2callback, function(callback, event) {
					framework[context].attachEvent(event, callback);
				});
			}
		);
	},
	
	selectById : function(tabId) {
		tabId = tabId || this.currentTabId;
		
		var tab = this.findWhere({
			id : tabId
		});
		
		return tab;
	}
	
});